﻿using System;

namespace EonaCat.Quic.Helpers
{
    // This file is part of the EonaCat project(s) which is released under the Apache License.
    // See the LICENSE file or go to https://EonaCat.com/License for full license details.

    public class IntegerVar
    {
        public const UInt64 MaxValue = 4611686018427387903;

        private UInt64 _integer;

        public UInt64 Value
        { get { return _integer; } }

        public IntegerVar(UInt64 integer)
        {
            _integer = integer;
        }

        public static implicit operator byte[](IntegerVar integer)
        {
            return Encode(integer._integer);
        }

        public static implicit operator IntegerVar(byte[] bytes)
        {
            return new IntegerVar(Decode(bytes));
        }

        public static implicit operator IntegerVar(UInt64 integer)
        {
            return new IntegerVar(integer);
        }

        public static implicit operator UInt64(IntegerVar integer)
        {
            return integer._integer;
        }

        public static implicit operator IntegerVar(StreamId streamId)
        {
            return new IntegerVar(streamId.IntegerValue);
        }

        public static int Size(byte firstByte)
        {
            int result = (int)Math.Pow(2, (firstByte >> 6));

            return result;
        }

        public byte[] ToByteArray()
        {
            return Encode(this._integer);
        }

        public static byte[] Encode(UInt64 integer)
        {
            int requiredBytes = 0;
            if (integer <= byte.MaxValue >> 2) /* 63 */
                requiredBytes = 1;
            else if (integer <= UInt16.MaxValue >> 2) /* 16383 */
                requiredBytes = 2;
            else if (integer <= UInt32.MaxValue >> 2) /* 1073741823 */
                requiredBytes = 4;
            else if (integer <= UInt64.MaxValue >> 2) /* 4611686018427387903 */
                requiredBytes = 8;
            else
                throw new ArgumentOutOfRangeException("Value is larger than IntegerVar.MaxValue.");

            int offset = 8 - requiredBytes;

            byte[] uInt64Bytes = ByteHelpers.GetBytes(integer);
            byte first = uInt64Bytes[offset];
            first = (byte)(first | (requiredBytes / 2) << 6);
            uInt64Bytes[offset] = first;

            byte[] result = new byte[requiredBytes];
            Buffer.BlockCopy(uInt64Bytes, offset, result, 0, requiredBytes);

            return result;
        }

        public static UInt64 Decode(byte[] bytes)
        {
            int i = 8 - bytes.Length;
            byte[] buffer = new byte[8];

            Buffer.BlockCopy(bytes, 0, buffer, i, bytes.Length);
            buffer[i] = (byte)(buffer[i] & (255 >> 2));

            UInt64 res = ByteHelpers.ToUInt64(buffer);

            return res;
        }
    }
}