﻿using System;
using EonaCat.Quic.Helpers;
using EonaCat.Quic.Infrastructure.Frames;
using EonaCat.Quic.Infrastructure.Packets;

namespace EonaCat.Quic.Infrastructure.PacketProcessing
{
    // This file is part of the EonaCat project(s) which is released under the Apache License.
    // See the LICENSE file or go to https://EonaCat.com/License for full license details.

    public class PacketCreator
    {
        private readonly NumberSpace _ns;
        private readonly IntegerParts _connectionId;
        private readonly IntegerParts _peerConnectionId;

        public PacketCreator(IntegerParts connectionId, IntegerParts peerConnectionId)
        {
            _ns = new NumberSpace();

            _connectionId = connectionId;
            _peerConnectionId = peerConnectionId;
        }

        public ShortHeaderPacket CreateConnectionClosePacket(ErrorCode code, byte frameType, string reason)
        {
            ShortHeaderPacket packet = new ShortHeaderPacket(_peerConnectionId.Size);
            packet.PacketNumber = _ns.Get();
            packet.DestinationConnectionId = (byte)_peerConnectionId;
            packet.AttachFrame(new ConnectionCloseFrame(code, frameType, reason));

            return packet;
        }

        public ShortHeaderPacket CreateDataPacket(UInt64 streamId, byte[] data, UInt64 offset, bool eos)
        {
            ShortHeaderPacket packet = new ShortHeaderPacket(_peerConnectionId.Size);
            packet.PacketNumber = _ns.Get();
            packet.DestinationConnectionId = (byte)_peerConnectionId;
            packet.AttachFrame(new StreamFrame(streamId, data, offset, eos));

            return packet;
        }

        public InitialPacket CreateServerBusyPacket()
        {
            return new InitialPacket(0, 0);
        }

        public ShortHeaderPacket CreateShortHeaderPacket()
        {
            return new ShortHeaderPacket(0);
        }
    }
}