﻿using System;
using System.Collections.Generic;
using EonaCat.Quic.Helpers;
using EonaCat.Quic.Infrastructure.Exceptions;
using EonaCat.Quic.Infrastructure.Frames;
using EonaCat.Quic.Infrastructure.Settings;

namespace EonaCat.Quic.Infrastructure.Packets
{
    // This file is part of the EonaCat project(s) which is released under the Apache License.
    // See the LICENSE file or go to https://EonaCat.com/License for full license details.

    /// <summary>
    /// Base data transfer unit of QUIC Transport.
    /// </summary>
    public abstract class Packet
    {
        protected List<Frame> _frames = new List<Frame>();
        public abstract byte Type { get; }

        public UInt32 Version { get; set; }

        public abstract byte[] Encode();

        public abstract void Decode(byte[] packet);

        public virtual void AttachFrame(Frame frame)
        {
            _frames.Add(frame);
        }

        public virtual List<Frame> GetFrames()
        {
            return _frames;
        }

        public virtual void DecodeFrames(ByteArray array)
        {
            FrameParser factory = new FrameParser(array);
            Frame result;
            int frames = 0;
            while (array.HasData() && frames <= QuicSettings.PMTU)
            {
                result = factory.GetFrame();
                if (result != null)
                    _frames.Add(result);

                frames++;
            }

            if (array.HasData())
                throw new ProtocolException("Unexpected number of frames or possibly corrupted frame was sent.");
        }

        public virtual byte[] EncodeFrames()
        {
            List<byte> result = new List<byte>();
            foreach (Frame frame in _frames)
            {
                result.AddRange(frame.Encode());
            }

            return result.ToArray();
        }
    }
}