﻿using System.Net;
using System.Net.Sockets;
using EonaCat.Quic.Exceptions;
using EonaCat.Quic.Infrastructure.Packets;

namespace EonaCat.Quic.InternalInfrastructure
{
    // This file is part of the EonaCat project(s) which is released under the Apache License.
    // See the LICENSE file or go to https://EonaCat.com/License for full license details.

    internal class PacketWireTransfer
    {
        private UdpClient _client;
        private IPEndPoint _peerEndpoint;

        private Unpacker _unpacker;

        public PacketWireTransfer(UdpClient client, IPEndPoint peerEndpoint)
        {
            _client = client;
            _peerEndpoint = peerEndpoint;

            _unpacker = new Unpacker();
        }

        public Packet ReadPacket()
        {
            // Await response for sucessfull connection creation by the server
            byte[] peerData = _client.Receive(ref _peerEndpoint);
            if (peerData == null)
                throw new ConnectionException("Server did not respond properly.");

            Packet packet = _unpacker.Unpack(peerData);

            return packet;
        }

        public bool SendPacket(Packet packet)
        {
            byte[] data = packet.Encode();

            int sent = _client.Send(data, data.Length, _peerEndpoint);

            return sent > 0;
        }

        public IPEndPoint LastTransferEndpoint()
        {
            return _peerEndpoint;
        }
    }
}