﻿using System.Collections.Generic;
using System.IO;
using System.Text;

    // This file is part of the EonaCat project(s) which is released under the Apache License.
    // See the LICENSE file or go to https://EonaCat.com/License for full license details.

namespace EonaCat.Network
{
    public class FileReader
    {
        /// <summary>
        /// Read a configuration file (varName=varValue format #beginning with comments)
        /// </summary>
        /// <returns>Parameter list.</returns>
        /// <param name="filePath">File path.</param>
        public static Dictionary<string, string> ReadFile(string filePath)
        {
            FileReader handler = new FileReader(filePath);

            Dictionary<string, string> result = handler.Read();

            handler = null;

            return result;
        }

        private readonly StreamReader streamReader;

        public FileReader(string filePath) : this(filePath, NetworkHelper.GlobalEncoding)
        {
        }

        public FileReader(string filePath, Encoding codingType)
        {
            streamReader = new StreamReader(filePath, codingType);
        }

        // return parameters
        public Dictionary<string, string> Read()
        {
            Dictionary<string, string> result = new Dictionary<string, string>();

            string line;

            while ((line = streamReader.ReadLine()) != null)
            {
                if (line.Length > 0)
                {
                    if (line.Substring(0, 1) == @"#")
                    {
                        line = "";

                        continue;  // annotation line
                    }
                }
                else // empty line
                {
                    continue;
                }

                string[] strPair = line.Split('=');

                line = "";

                if (strPair.Length == 2)
                {
                    result.Add(
                        strPair[0].ToUpper().Replace("\"", "").Replace("\'", "").TrimStart().TrimEnd(),
                        strPair[1].Replace("\"", "").Replace("\'", "").TrimStart().TrimEnd());
                }
                else
                {
                    continue;
                }
            }

            streamReader.Close();

            return result;
        }
    }
}