﻿using System.IO;

namespace EonaCat.Network
{
    // This file is part of the EonaCat project(s) which is released under the Apache License.
    // See the LICENSE file or go to https://EonaCat.com/License for full license details.

    public class FileWriter
    {
        /// <summary>
        /// Write a file
        /// </summary>
        /// <param name="filePath">File address</param>
        /// <param name="content">Write content.</param>
        public static void WriteFile(string filePath, string content)
        {
            FileWriter handler = new FileWriter(filePath);

            handler.Write(content, false);

            handler.Finished();

            handler = null;
        }

        private readonly FileStream fileStream;
        private readonly StreamWriter streamWriter;

        public FileWriter(string filePath)
        {
            fileStream = new FileStream(filePath, FileMode.OpenOrCreate);

            streamWriter = new StreamWriter(fileStream);
        }

        public void Write(string content, bool isLine)
        {
            if (isLine)
            {
                streamWriter.WriteLine(content);
            }
            else
            {
                streamWriter.Write(content);
            }

            streamWriter.Flush();
        }

        public void Finished()
        {
            streamWriter.Close();

            fileStream.Close();
        }
    }
}